/* Sets normal, position, color per vertex */
uniform mat4 uProjMatrix;
uniform mat4 uViewMatrix;
uniform mat4 uModelMatrix;
uniform mat4 uNormalMatrix;
uniform vec3 uColor;
uniform vec3 uLight;
uniform vec3 uCamera;
uniform float uDiffuseCoefficient;
uniform float uSpecularCoefficient;
uniform float uSpecularHardness;

attribute vec3 aPosition;
attribute vec3 aNormal;

varying vec4 vPosition;
varying vec3 vNormal;
varying vec3 vColor;

void main() { 
    vPosition = uModelMatrix * vec4(aPosition, 1.0);
    vec4 Normal = vec4(aNormal, 1.0);
    vNormal = normalize((uNormalMatrix * Normal).xyz);
    
    vColor = uColor; // No per-vertex colors
  
    gl_Position = uProjMatrix * (uViewMatrix * vPosition);
    
    float diffuse = 0.0;
    float specular = 0.0;
  
    // Pretend to be obscured by ground
    if(uLight.y >= 0.0) {
        vec3 N = normalize(vNormal);
        vec3 L = normalize(uLight - vec3(vPosition.x, vPosition.y, vPosition.z));  
        diffuse = uDiffuseCoefficient * max(0.0, dot(N, L));
        
        vec3 R = normalize((-1.0*L) + ((2.0 * dot(L, N)) * N));
        vec3 V = normalize(uCamera - vec3(vPosition.x, vPosition.y, vPosition.z));  
        specular = uSpecularCoefficient * pow(max(0.0, dot(V, R)), uSpecularHardness);
    }
     
    // simulate sunrise/sunset
    float lightPct = clamp(uLight.y * 0.1, 0.1, 1.0);
    float ambient = clamp(lightPct + .1, 0.25, 0.4);
    
    /* ambient + diffuse + specular */
    vec3 litColor;
    litColor.r = clamp(vColor.r*(ambient + (diffuse + specular)*lightPct), 0.0, 1.0);
    litColor.g = clamp(vColor.g*(ambient + (diffuse + specular)*lightPct), 0.0, 1.0);
    litColor.b = clamp(vColor.b*(ambient + (diffuse + specular)*lightPct), 0.0, 1.0);
      
    vColor = litColor;
}
