/*
 * CameraHelper.cpp
 *
 *  Created on: Nov 14, 2012
 *      Author: Karl
 */

#include "CameraHelper.h"

CameraHelper::CameraHelper() {
    reset();
}

CameraHelper::~CameraHelper() {
}

void CameraHelper::reset() {
    offset = vec3(0);
    pitch = 0.0f;
    yaw = START_YAW;
}

glm::vec3 CameraHelper::getRelativeLookPoint() {
    return glm::vec3(cos(pitch) * cos(yaw), sin(pitch), cos(pitch) * cos(YAW_OFF - yaw));
}

glm::vec3 CameraHelper::getEye() {
    return offset;
}

glm::vec3 CameraHelper::getCenter() {
    return offset + getRelativeLookPoint();
}

void CameraHelper::addYaw(double x) {
    yaw += x;
    // correct to within -PI to PI
    while (yaw > PI) {
        yaw -= 2 * PI;
    }
    while (yaw < -PI) {
        yaw += 2 * PI;
    }
}

void CameraHelper::addPitch(double y) {
    pitch = std::min(MAX_PITCH, std::max(MIN_PITCH, pitch + y));
}

void CameraHelper::move(double move) {
    vec3 w = glm::normalize(getRelativeLookPoint());

    offset += vec3(w.x * move, 0, w.z * move);
}

void CameraHelper::strafe(double strafe) {
    vec3 w = glm::normalize(getRelativeLookPoint());
    vec3 u = glm::normalize(glm::cross(vec3(0, 1, 0), w));
    vec3 v = glm::normalize(glm::cross(w, u));

    offset += vec3(u.x * strafe, 0, u.z * strafe);
}

void CameraHelper::fly(double fly) {
    offset.y += fly;
}
