/*
 * ModelHelper.h
 *
 *  Created on: Oct 18, 2012
 *      Author: Karl
 */

#ifndef MODELHELPER_H_
#define MODELHELPER_H_

#include "gl.h"

#include "glm/glm.hpp"
#include "glm/gtx/transform.hpp"
#include "glm/gtc/type_ptr.hpp"
#include "GeometryCreator.h"
#include "MStackHelp.h"
#include "GLSL_helper.h"
#include "types.h"
#include <algorithm>

using glm::vec3;
using glm::mat4;

#define S_TO_MILLIS 1000
#define MILLIS_TO_S (1.0 / S_TO_MILLIS)

#define MAX_TRANS 2000.0f
#define MIN_TRANS -2000.0f
#define MAX_SCALE 2.5f
#define MIN_SCALE 0.1f
#define MIN_VEL -2000.0f
#define MAX_VEL 2000.0f

class ModelHelper;

class ModelHelper {
public:

    float xScale, yScale, zScale;
    vec3 offset;
    mat4 rotMatrix;

    bool fixed;
    vec3 velocity;
    float elasticity;
    float mass;

    bool sphere; // sphere or plane
    float r; // sphere
    vec3 n; // plane

    vec3 color;
    float diffuseCoefficient;
    float specularCoefficient;
    float specularHardness;

    struct Mesh *mesh;

    vec3 attachment;
    std::vector<ModelHelper*> children;

    ModelHelper() {
        init();
    }
    ModelHelper(Mesh *m, vec3 c) :
            mesh(m), color(c) {
        init();
        attachment = vec3(0);
        children = std::vector<ModelHelper*>();
    }
    ModelHelper(Mesh *m, vec3 c, vec3 attach) :
            mesh(m), color(c), attachment(attach) {
        init();
        children = std::vector<ModelHelper*>();
    }
    ModelHelper(Mesh *m, vec3 c, vec3 attach, std::vector<ModelHelper*> ch) :
            mesh(m), color(c), attachment(attach), children(ch) {
        init();
    }
    ~ModelHelper() {
    }

    void init();
    void reset();

    void addTranslate(float x, float y, float z);
    void addTranslate(vec3 t);
    void addScale(float x, float y, float z);
    void addRotation(vec3 axis, float angle);
    void addVelocity(vec3 vel);
    void addVelocity(float x, float y, float z);

    void tick(int millis);

    bool intersect(ModelHelper & o, vec3 * myIntersect, vec3 * oIntersect, vec3 * normal);

    void draw(RenderingHelper & ModelTrans, struct glHandles & h);
    void recalculateBB();

};

#endif /* MODELHELPER_H_ */
