/*
 * PhysicsEngine.cpp
 *
 *  Created on: Nov 20, 2012
 *      Author: Karl
 */

#include "PhysicsEngine.h"

bool randomBool() {
   return rand() % 2 == 0;
}

float randomFloat(float lo, float hi) {
   return lo + (float)rand()/((float)RAND_MAX/(hi-lo));
}

vec3 addNoise(vec3 v) {
   v.x += randomFloat(NOISE_MIN, NOISE_MAX) * (randomBool() ? 1 : -1);
   v.y += randomFloat(NOISE_MIN, NOISE_MAX) * (randomBool() ? 1 : -1);
   v.z += randomFloat(NOISE_MIN, NOISE_MAX) * (randomBool() ? 1 : -1);
   return v;
}

void decompose(vec3 vec, vec3 dir, vec3 * vdir, vec3 * vperp) {
   vec3 d = glm::normalize(addNoise(glm::normalize(dir)));
   glm::float_t ang = std::max(0.0f, glm::dot(glm::normalize(vec), d));
   glm::float_t dpart = glm::distance(vec3(0, 0, 0), vec) * ang;
   vec3 vd = dpart * d; // component in vector dir
   vec3 vp = vec - vd; // component perpendicular to vector

   *vdir = vd;
   *vperp = vp;
}

void PhysicsEngine::tick(int millis) {
   for (std::vector<ModelHelper>::iterator it = models.begin(); it != models.end(); ++it) {
      it->addVelocity(gravity * (glm::float_t)millis);
   }

   for (std::vector<ModelHelper>::iterator it = models.begin(); it != models.end(); ++it) {
      it->tick(millis);
   }

   // Check collisions, untick, modify velocities
   // FIXME multiple collisions (find collision groups, resolve order, simulate)
   for (int i = models.size() - 1; i > 0; i--) {
      ModelHelper & m1 = models[i];
      bool flag = false;
      for (int j = 0; j < i; j++) {
         vec3 i1 = vec3(0, 0, 0);
         vec3 i2 = vec3(0, 0, 0);
         vec3 n = vec3(0, 0, 0);
         ModelHelper & m2 = models[j];

         if (m1.fixed && m2.fixed) {
            continue;
         }

         if (m1.intersect(m2, &i1, &i2, &n)) {
            flag = true;
            int millisLeft = 0;
            do {
               m1.tick(-COLLISION_RES);
               //m1.addVelocity(vec3(0, GRAVITY * COLLISION_RES, 0));
               m2.tick(-COLLISION_RES);
               //m2.addVelocity(vec3(0, GRAVITY * COLLISION_RES, 0));
               millisLeft += COLLISION_RES;
            } while (m1.intersect(m2, &i1, &i2, &n) && millisLeft < millis);

            if (m1.fixed ^ m2.fixed) {
               ModelHelper & m = m1.fixed ? m2 : m1;
               vec3 & i = m1.fixed ? i2 : i1;
               vec3 cv, pv;
               decompose(m.velocity, i, &cv, &pv);
      
               m.velocity = ((cv * (glm::float_t)(-m.elasticity)) + pv * (glm::float_t)(1.0 / FRICTION));
            } else if (!m1.fixed && !m2.fixed) {
               // real collision
               vec3 cv1, cv2, pv1, pv2;
               decompose(m1.velocity, i1, &cv1, &pv1);
               decompose(m2.velocity, i2, &cv2, &pv2);

               // swap velocities in collision vector terms, add back in perpendicular velocities
               m1.velocity = ((cv2 * (glm::float_t)(m2.mass / m1.mass * m1.elasticity)) + pv1);
               m2.velocity = ((cv1 * (glm::float_t)(m1.mass / m2.mass * m2.elasticity)) + pv2);
            }
            m1.tick(millisLeft);
            m2.tick(millisLeft);
         }
      }
      if (!flag) {
      }
   }
}

